def print_red(skk): 
    print("\033[91m{}\033[00m" .format(skk)) 


def print_green(skk): 
    print("\033[92m{}\033[00m" .format(skk)) 


def determine_print_color_from_prices(price, purchase_price):
    print_color = print_green if price > purchase_price else print_red
    return print_color


def info_str(day, action, ticker, volume, price):
    return f'{day}: {action:<{2}} {ticker} \t {volume:>{4}} \t for {round(price, 2)}'



from marketools.stock import *
from marketools.wallet import Wallet
from marketools.simulator import simulator, Simulator
from marketools.analysis import *


__pdoc__ = dict()
__pdoc__['extra_print'] = False
__pdoc__['wallet'] = False



import pandas as pd


def simple_relative_price_change(new_price: float, ref_price: float):
    """
    Calculates and returns relative price change.

    Parameters
    ----------
    new_price : float
        new/current price
    ref_price : float
        reference price

    Returns
    -------
    float
    """
    
    assert ref_price > 0
    return (new_price - ref_price) / ref_price


def price_change(ohlc: pd.DataFrame, 
                 shift: int = 0,
                 relative: bool = False,
                 percent: bool = False):
    """
    Calculates price change based on given DataFrame with OHLC prices.

    Parameters
    ----------
    ohlc : pandas.DataFrame
        DataFrame with OHLC data
    shift : int
        default is zero - price change is calculated as change within one day 
        (Close-Open); if greater than zero, the change is calculated in relation 
        to close price "shift" days before (Close[today]-Close[today-shift]) 
    relative : bool
        if True calculates relative price change
    relative : bool
        if True returns relative price change expressed in percentage (relative 
        must be True)

    Returns
    -------
    pandas.Series
    """

    if shift:
        ref_price = ohlc['Close'].shift(shift)
        name_str = f'({shift}d)'
    else:
        ref_price = ohlc['Open']
        name_str = '(daily)'

    change = ohlc['Close'] - ref_price

    if relative:
        change = change / ref_price
        if percent:
            change = 100 * change
            name_str = '%Change ' + name_str
        else:
            name_str = 'Relative change ' + name_str
    else:
        name_str = 'Change ' + name_str

    change = change.rename(name_str)

    return change



from marketools.analysis.rsi import relative_strength_index as rsi
from marketools.analysis.rsi import rsi_cross_signals
from marketools.analysis.macd import macd
from marketools.analysis.moving_average import simple_moving_average as sma
from marketools.analysis.moving_average import weighted_moving_average as wma
from marketools.analysis.moving_average import exponential_moving_average as ema
from marketools.analysis.price import simple_relative_price_change, price_change
from marketools.analysis.volume import mean_volume_on_date


relative_price_change = simple_relative_price_change





def mean_volume_on_date(volume_data, 
                        day, 
                        window=90):
    """
    Returns mean volume over given number of sessions before given date 
    (inclusively).

    Parameters
    ----------
    volume_data : pandas.DataFrame
        DataFrame with 'Volume' column
    day : date 
        date the mean volume should be calculated for
    window : int
        number of recent stock market sessions the average should be calculated 
        over

    Returns
    -------
    float
    """
    
    volume = volume_data[:day].tail(window)['Volume']
    output = volume.mean()
    
    return output



import math
import warnings
import pandas as pd
from marketools.wallet import calculate_investment_value, Wallet
from marketools.extra_print import print_green, print_red, \
    determine_print_color_from_prices, info_str


class Simulator:
    """
    Simulator for stock market strategies.

    Attributes
    ----------
    time_range : pandas.DatetimeIndex
        an array with trading days only (no Saturdays, Sundays, holidays)
    traded_stocks_data : dict
        a dictionary with stocks.stock.Stock instances
    wallet : marketools.Wallet
        Wallet for trading
    max_positions : int
        maximum number of different stocks in the wallet
    take_profit : float
        if the price of a stock increases by this fraction, it will be sold;
        if equal 0 - take profit is deactivated (default)
    stop_loss : float
        if the price of a stock decreases by this fraction (comparing to the
        purchase price), it will be sold;
        if equal 0 - stop loss is deactivated (default)
    live_trading : boolean
        if True selling immediately when stop loss / take profit is reached is
        simulated
    """

    def __init__(self, time_range: pd.DatetimeIndex, traded_stocks_data: dict,
                 wallet: Wallet, max_positions: int = 5,
                 take_profit: float = 0.0, stop_loss: float = 0.0,
                 live_trading: bool = False):

        self.time_range = time_range
        self.traded_stocks_data = traded_stocks_data
        self.wallet = wallet
        self.wallet_init_value = wallet.total_value
        self.max_positions = max_positions
        self.take_profit = take_profit
        self.stop_loss = stop_loss
        self.live_trading = live_trading
        self.wallet_history = pd.DataFrame(columns=['Date', 'Wallet state'])

    def reset(self) -> None:
        """
        Resets the value of wallet to initial value, and removes all stocks from
        the wallet and its history.
        """
        self.wallet = Wallet(self.wallet.rate, self.wallet.minimum)
        self.wallet.money = self.wallet_init_value
        self.wallet_history = pd.DataFrame(columns=['Date', 'Wallet state'])

    def run(self, strategy_function, *args, **kwargs) -> pd.DataFrame:
        """
        Runs the simulation for given strategy and returns DataFrame with
        wallet value for each day in given time range.

        Parameters
        ----------
        strategy_function : func
            Function with investment strategy. Must take dictionary with traded
            stocks data (self.traded_stocks_data) and date as inputs. Must
            return set of lists (stocks_to_buy, stocks_to_sell).
        kwargs
            Other inputs to investment strategy (strategy_function).

        Returns
        -------
        pandas.DataFrame
        """

        stocks_to_buy = []
        stocks_to_sell = []

        for day in self.time_range:
            day_str = day.strftime('%Y-%m-%d')

            # Buy selected day before. Loop over list, order can be important
            # here. Strategy can sort relevant stocks - high priority first.
            for tck in stocks_to_buy:

                if not self.wallet.get_volume_of_stocks(tck):
                    price = self.traded_stocks_data[tck].ohlc['Open']\
                        .get(day, None)

                    if price:
                        total = calculate_investment_value(
                            self.wallet, self.max_positions)

                        # needs some money to pay commission
                        total = total - self.wallet.commission(total)
                        volume = math.floor(total / price)

                        if volume > 0:
                            self.wallet.buy(tck, volume, price)
                            print(info_str(day_str, 'B', tck, volume, price))
                            # make sure to not sell it the same day

                            if tck in stocks_to_sell:
                                stocks_to_sell.remove(tck)

            # Sell selected day before. List to set, we don't care here about
            # the order. Set will remove duplicates.
            for tck in set(stocks_to_sell):

                if self.wallet.get_volume_of_stocks(tck):
                    price = self.traded_stocks_data[tck].ohlc['Open']\
                        .get(day, None)

                    if price:
                        print_color = determine_print_color_from_prices(
                            price,
                            self.wallet.get_purchase_price_of_stocks(tck))
                        volume = self.wallet.sell_all(tck, price)
                        print_color(info_str(day_str, 'S', tck, volume, price))

            # call decorated function - strategy function
            stocks_to_buy, stocks_to_sell = strategy_function(
                day=day, traded_stocks=self.traded_stocks_data, *args, **kwargs)

            # if auto trading is active, check if price of any stock in the
            # wallet crossed take profit or stop loss price; if yes then sell it
            # immediately
            if self.live_trading:

                for tck in self.wallet.list_stocks().copy():
                    price_max = self.traded_stocks_data[tck].ohlc['High']\
                        .get(day, None)

                    if price_max:
                        self.wallet.update_price(tck, price_max)

                    if self.wallet.change(tck) > self.take_profit:
                        # selling it immediately
                        price = self.wallet.get_purchase_price_of_stocks(tck)\
                                * (1 + self.take_profit)
                        price = round(price, 2)
                        volume = self.wallet.sell_all(tck, price)
                        print_green(info_str(day_str, 'TP', tck, volume, price))
                        continue

                    price_min = self.traded_stocks_data[tck].ohlc['Low']\
                        .get(day, None)

                    if price_min:
                        self.wallet.update_price(tck, price_min)

                    if self.wallet.change(tck) < -self.stop_loss:
                        # selling it immediately
                        price = self.wallet.get_purchase_price_of_stocks(tck)\
                                * (1 - self.stop_loss)
                        price = round(price, 2)
                        volume = self.wallet.sell_all(tck, price)
                        print_red(info_str(day_str, 'SL', tck, volume, price))

            for tck in self.wallet.list_stocks():
                # update the price to the closing price
                ohlc = self.traded_stocks_data[tck].ohlc
                price = ohlc['Close'].get(day, None)

                if price:
                    self.wallet.update_price(tck, price)

                # if auto trading is not active, then take profit / stop loss
                # the next day
                if not self.live_trading:

                    # take profit the next day
                    if (self.take_profit > 0) \
                            and (self.wallet.change(tck) > self.take_profit):
                        stocks_to_sell.append(tck)

                    # stop loss the next day - price below purchase price
                    if (self.stop_loss > 0)\
                            and (self.wallet.change(tck) < -self.stop_loss):
                        stocks_to_sell.append(tck)

            # save history of the wallet
            self.wallet_history = self.wallet_history.append(
                {'Date': day, 'Wallet state': self.wallet.total_value},
                ignore_index=True)

        return self.wallet_history


def simulator(time_range: pd.DatetimeIndex,
              traded_stocks: dict, 
              wallet: Wallet, 
              max_positions: int = 5, 
              take_profit: float = 0.0, 
              stop_loss: float = 0.0, 
              auto_trading: bool = False):
    """
    This function returns decorator for a stock market strategy, that returns 
    change of wallet value over given time_range (in pandas.DataFrame)

    Parameters
    ----------
    time_range : pandas.DatetimeIndex
        an array with trading days only (no Saturdays, Sundays, holidays)
    traded_stocks : dict
        a dictionary with stocks.stock.Stock instances
    wallet : marketools.Wallet
        Wallet for trading 
    max_positions : int 
        maximum number of different stocks in the wallet
    take_profit : float
        if the price of a stock increases by this fraction, it will be sold;
        if equal 0 - take profit is deactivated (default)
    stop_loss : float
        if the price of a stock decreases by this fraction (comparing to the 
        purchase price), it will be sold; 
        if equal 0 - stop loss is deactivated (default)
    auto_trading : boolean
        if True selling immediately when stop loss / take profit is reached is 
        simulated

    Returns
    -------
    decorator that returns pandas.DataFrame
    """

    warnings.warn("deprecated, consider using Simulator class instead",
                  DeprecationWarning)

    def strategy_wrapper(func):

        def simulation(*args, **kwargs):
            wallet_history = pd.DataFrame(columns=['Date', 'Wallet state'])
            stocks_to_buy = []
            stocks_to_sell = []

            for day in time_range:
                day_str = day.strftime('%Y-%m-%d')

                # Buy selected day before. Loop over list, order can be important here.
                # Strategy can sort relevant stocks - high priority first.
                for tck in stocks_to_buy:
                    if not wallet.get_volume_of_stocks(tck):
                        price = traded_stocks[tck].ohlc['Open'].get(day, None)
                        if price:
                            total = calculate_investment_value(wallet, max_positions)
                            total = total - wallet(total)  # needs some money to pay commission
                            volume = math.floor(total/price)
                            if volume > 0:
                                wallet.buy(tck, volume, price)
                                print(info_str(day_str, 'B', tck, volume, price))
                                # make sure to not sell it the same day
                                if tck in stocks_to_sell:
                                    stocks_to_sell.remove(tck)

                # Sell selected day before. List to set, we dont care here about the order.
                # Set will remove duplicates.
                for tck in set(stocks_to_sell):
                    if wallet.get_volume_of_stocks(tck):
                        price = traded_stocks[tck].ohlc['Open'].get(day, None)
                        if price: 
                            print_color = determine_print_color_from_prices(price, wallet.get_purchase_price_of_stocks(tck))
                            volume = wallet.sell_all(tck, price)
                            print_color(info_str(day_str, 'S', tck, volume, price))

                # call decorated function - strategy function
                stocks_to_buy, stocks_to_sell = func(day=day, traded_stocks=traded_stocks, *args, **kwargs)

                # if auto trading is active, check if price of any stock in the wallet crossed
                # take proffit or stop loss price; if yes then sell it immediately 
                if auto_trading:
                    for tck in wallet.list_stocks().copy():

                        price_max = traded_stocks[tck].ohlc['High'].get(day, None)
                        if price:
                            wallet.update_price(tck, price_max)
                        if wallet.change(tck) > take_profit:
                            price = wallet.get_purchase_price_of_stocks(tck) * (1+take_profit)  # selling it immediately
                            price = round(price, 2)
                            volume = wallet.sell_all(tck, price)
                            print_green(info_str(day_str, 'TP', tck, volume, price))
                            continue

                        price_min = traded_stocks[tck].ohlc['Low'].get(day, None)
                        if price:
                            wallet.update_price(tck, price_min)
                        if wallet.change(tck) < -stop_loss:
                            price = wallet.get_purchase_price_of_stocks(tck) * (1-stop_loss)  # selling it immediately
                            price = round(price, 2)
                            volume = wallet.sell_all(tck, price)
                            print_red(info_str(day_str, 'SL', tck, volume, price))
                            
                for tck in wallet.list_stocks():                        
                    # update the price to the closing price and calculate relative price change
                    ohlc = traded_stocks[tck].ohlc
                    price = ohlc['Close'].get(day, None)
                    if price:
                        wallet.update_price(tck, price)

                    # if auto trading is not active, then take profit / stop loss the next day
                    if not auto_trading:

                        # take profit the next day
                        if take_profit and wallet.change(tck) > take_profit:
                            stocks_to_sell.append(tck)

                        # stop loss the next day - price below purchase price
                        if stop_loss and wallet.change(tck) < -stop_loss:
                            stocks_to_sell.append(tck)

                # save history of the wallet
                wallet_history = wallet_history.append(
                    {'Date': day, 'Wallet state': wallet.total_value},
                    ignore_index=True
                    )
            
            return wallet_history

        return simulation

    return strategy_wrapper


# if None:
#     """
#     This code should not be called - this is just a template function for tested strategy.
#     """
#     @simulator(TRADING_DAYS, stocks_data, MY_WALLET, TAKE_PROFIT, STOP_LOSS)
#     def __strategy_template(arguments, *args, **kwargs):
#         """
#         :param arguments: any arguments needed for the strategy can be passed
#         """
#         day = kwargs['day']  # argument passed by decorator
#         traded_stocks = kwargs['traded_stocks']  # argument passed by decorator
#         stocks_to_buy = []
#         stocks_to_sell = []
#
#         """
#         place for code that will fill in stocks_to_buy and stocks_to_sell sets with tickers
#         """
#
#         return stocks_to_buy, stocks_to_sell


if __name__ == '__main__':
    pass



import pandas as pd


def macd(prices: pd.DataFrame, 
         mid_const: int = 12, 
         long_const: int = 26, 
         signal_const: int = 9):
    """
    Returns Pandas Dataframe with MACD, Signal, and Histogram.

    Parameters
    ----------
    prices : pandas.DataFrame
        DataFrame with OHLC data (must have column 'Close')
    mid_const : int
        period for fast exponential moving average
    long_const : int
        period for slow exponential moving average
    signal_const : int
        period for signal exponential moving average
        
    Returns
    -------
    pandas.DataFrame
    """

    price = prices['Close']
    output = pd.DataFrame(columns=['MACD', 'Signal', 'Histogram'])

    # calculate MACD line = price.emw(12) - price.emw(26)
    ewm_mid = price.ewm(span=mid_const).mean()
    ewm_long = price.ewm(span=long_const).mean()
    output['MACD'] = ewm_mid - ewm_long

    # calculate signal line = MACD.emw(9)
    output['Signal'] = output['MACD'].ewm(span=signal_const).mean()

    # calculate histogram = MACD - signal
    output['Histogram'] = output['MACD'] - output['Signal']
    
    return output


if __name__=='__main__':
    pass



import os


# DATA_DIR = os.path.join(os.path.expanduser('~'), '.stock_market_data')
DATA_DIR = os.path.join(os.getcwd(), '.stock_market_data')

if not os.path.exists(DATA_DIR):
    os.mkdir(DATA_DIR)



import pandas as pd


class Commission:
    def __init__(self, rate: float, minimum: float):
        if minimum < 0:
            raise ValueError('minimum has to be non-negative value')
        if rate <= 0:
            raise ValueError('rate has to be positive value')

        self.rate = rate
        self.minimum = minimum

    def __call__(self, trade: float, *args, **kwargs) -> float:
        fee = trade * self.rate
        fee = round(fee, 2)
        return max(fee, self.minimum)

    def minimal_recommended_investment(self):
        return self.minimum / self.rate


class Wallet(Commission):
    def __init__(self, commission_rate: float, min_commission):
        super().__init__(commission_rate, min_commission)
        self.money = 0
        self.stocks = pd.DataFrame(columns=['Name', 'Volume', 'Purchase price', 'Price'])

    @property
    def money(self) -> float:
        return self.__money

    @property
    def stocks(self) -> pd.DataFrame:
        return self.__stocks

    @property
    def stocks_value(self):
        values = self.stocks.loc[:, ('Volume', 'Price')].product(axis=1)
        return values.sum()

    @property
    def total_value(self):
        return self.stocks_value + self.money

    @money.setter
    def money(self, pay_in: float):
        self.__money = pay_in

    @stocks.setter
    def stocks(self, new_stocks: pd.DataFrame):
        self.__stocks = new_stocks

    def __str__(self) -> str:
        return f'Stocks: \n' \
               f'{self.stocks} \n\n' \
               f'Stocks value: \t {self.stocks_value} \n' \
               f'Money: \t\t\t {self.money} \n' \
               f'Total value: \t {self.total_value} \n'

    def __add__(self, other: pd.DataFrame):
        """

        :param other: pd.DataFrame with columns 'Name', 'Volume', 'Purchase price', 'Price'
        :return:
        """
        """ added values """
        name = other.loc[0, 'Name']
        bought = other.loc[0, 'Volume']
        price = other.loc[0, 'Purchase price']

        """ owned stocks with same name """
        in_wallet = self.get_volume_of_stocks(name)
        wallet_price = self.get_purchase_price_of_stocks(name)

        """ calculate cost """
        cost = bought * price
        cost += self(cost)

        """ enough money in wallet to buy? """
        if cost <= self.money:
            if not in_wallet:
                """ stocks not in wallet - append them """
                self.stocks = self.stocks.append(other, ignore_index=True)
            else:
                """ stocks in wallet - increase volume and calculate average purchase price """
                idx = self.__get_stocks_index(name)
                self.stocks.loc[idx, 'Volume'] += bought
                self.stocks.loc[idx, 'Price'] = price
                avg_price = (price * bought + wallet_price * in_wallet) / (bought + in_wallet)
                self.stocks.loc[idx, 'Purchase price'] = avg_price

            self.money -= cost

        return self

    def __sub__(self, other: pd.DataFrame):
        """

        :param other: pd.DataFrame with columns 'Name', 'Volume', 'Price'
        :return:
        """
        """ subtracted values """
        name = other.loc[0, 'Name']
        sold = other.loc[0, 'Volume']
        price = other.loc[0, 'Price']

        """ owned stocks with the same name """
        in_wallet = self.get_volume_of_stocks(name)

        if in_wallet >= sold > 0:
            idx = self.__get_stocks_index(name)
            if in_wallet == sold:
                """ all stocks from wallet solc - drop them """
                self.stocks = self.stocks.drop(idx)
            else:
                """ part of stocks from wallet sold - decrease volume """
                self.stocks.loc[idx, 'Volume'] -= sold
                self.stocks.loc[idx, 'Price'] = price

            """ calculate gain """
            gain = sold * price
            gain -= self(gain)
            self.money += gain

        return self

    def commission(self, value: float) -> float:
        return self(value)

    def __get_stocks_index(self, name: str):
        idx = self.stocks.index[self.stocks['Name'] == name].tolist()
        idx = idx[0] if idx else None
        return idx

    def get_volume_of_stocks(self, name: str) -> int:
        idx = self.__get_stocks_index(name)
        if idx is not None:
            return self.stocks.loc[idx, 'Volume']
        else:
            return 0

    def get_purchase_price_of_stocks(self, name: str):
        idx = self.__get_stocks_index(name)
        if idx is not None:
            return self.stocks.loc[idx, 'Purchase price']
        else:
            return None

    def buy(self, name: str, volume: int, price: float) -> None:
        stock = pd.DataFrame({'Name': [name],
                              'Volume': [volume],
                              'Purchase price': [price],
                              'Price': [price]})
        self.__add__(stock)

    def sell(self, name: str, volume: int, price: float) -> None:
        stock = pd.DataFrame({'Name': [name],
                              'Volume': [volume],
                              'Price': [price]})
        self.__sub__(stock)

    def sell_all(self, name: str, price: float) -> float:
        volume = self.get_volume_of_stocks(name)
        self.sell(name, volume, price)
        return volume

    def list_stocks(self) -> list:
        return self.stocks.loc[:, 'Name'].to_list()

    def update_price(self, name: str, price: float) -> None:
        idx = self.__get_stocks_index(name)
        if idx is not None:
            self.stocks.loc[idx, 'Price'] = price

    def change(self, name: str) -> float:
        idx = self.__get_stocks_index(name)
        purchase_price = self.stocks.loc[idx, 'Purchase price']
        price = self.stocks.loc[idx, 'Price']
        output = (price - purchase_price) / purchase_price
        return output


def calculate_investment_value(wallet, max_positions):
    output = 0
    min_value = wallet.minimal_recommended_investment()
    if wallet.money > min_value:
        max_value = wallet.total_value / max_positions
        output = max(max_value, min_value)
        output = min(output, wallet.money)
    return output
    


import pandas as pd


def relative_strength_index(prices: pd.DataFrame, window: int = 14):
    """
    Calculates Relative Strength Index (RSI).

    Parameters
    ----------
    prices : pandas.DataFrame 
        DataFrame with 'Close' column containing closing prices of stock
    window : int
        size of the moving window.

    Returns
    -------
    pandas.Series
    """
    
    price_now = prices['Close']
    price_prev = prices['Close'].shift(periods=1, fill_value=0)  # assign to each day price from the previous day

    price_changes = pd.DataFrame(columns=['Up', 'Down'])
    price_changes['Up'] = price_now - price_prev  # upward changes
    price_changes['Down'] = price_prev - price_now  # downward changes
    price_changes[price_changes < 0] = 0

    smma = price_changes.ewm(alpha=1/window, adjust=False).mean()  # smoothed moving averages, alpha = 1/N (not 2/(N+1))
    rs = smma['Up'] / smma['Down']

    output_rsi = -100 / (rs+1)
    output_rsi = output_rsi + 100

    output_rsi = output_rsi.rename('RSI')

    return output_rsi


def rsi_cross_signals(rsi_values: pd.Series, 
                      cross_line: float, 
                      direction: str='rise'):
    """
    Calculates buy/sell signals for given RSI signal line. Returns table with 
    True values for days when signal appears.

    Parameters
    ----------
    rsi_values : pandas.Series 
        DataFrame with RSI column
    cross_line : float
        signal threshold line, when signal line crosses this line signal is set
    direction : str
        direction the signal line should cross threshold line 
        ('rise' - signal increasing [default], 'fall' - signal decreasing)
    
    Returns
    -------
    pandas.Series
    """

    if not (0 < cross_line < 100):
        raise ValueError('cross_line takes values from 0 to 100')

    rsi_copy = pd.DataFrame()
    rsi_copy['RSI'] = rsi_values
    rsi_copy['RSI day before'] = rsi_values.shift(1)

    if 'rise' == direction:
        # True signal if RSI is increasing and crossing the threshold line
        output = (rsi_copy['RSI'] >= cross_line ) & (rsi_copy['RSI day before'] < cross_line)
    elif 'fall' == direction:
        # True signal if RSI is decreasing and crossing the threshold line
        output = (rsi_copy['RSI'] <= cross_line ) & (rsi_copy['RSI day before'] > cross_line)
    else:
        raise ValueError('wrong value for direction, must be "rise" or "fall"')

    output = output.rename(f'Cross signal ({cross_line} on {direction})')

    return output



import pandas as pd
import numpy as np


def simple_moving_average(ohlc: pd.DataFrame,
                          price: str = 'Close',
                          window: int = 15):
    """
    Returns Pandas Series with the Simple Moving Average for the indicated price. NaN will be placed for days
    where there is not enough previous data (depends on min_periods)

    Parameters
    ----------
    ohlc : pandas.DataFrame
        DataFrame with OHLC data
    price : str
        The price on which SMA will be calculated (Close by default)
    window : int
        Size of the moving window. This is the number of observations used for calculating the statistic

    Returns
    -------
    pandas.Series
    """

    # implement simple moving average
    output = ohlc[price].rolling(window=window).mean()
    output = output.rename(f'SMA{window}')

    return output


def weighted_moving_average(ohlc: pd.DataFrame,
                            price: str = 'Close',
                            window: int = 15):
    """
    Returns Pandas Series with the Weighted Moving Average for the indicated price. NaN will be placed for days
    where there is not enough previous data

    Parameters
    ----------
    ohlc : pandas.DataFrame
        DataFrame with OHLC data
    price : str
        The price on which WMA will be calculated (Close by default)
    window : int
        Size of the moving window. This is the number of observations used for calculating the statistic

    Returns
    -------
    pandas.Series
    """

    weights = np.arange(1, window + 1)

    # implement weighted moving average
    output = ohlc[price].rolling(window=window)\
        .apply(lambda prices: np.dot(prices, weights)/weights.sum(), raw=True)

    output = output.rename(f'WMA{window}')

    return output


def exponential_moving_average(ohlc: pd.DataFrame,
                               price: str = 'Close',
                               window: int = 15):
    """
    Returns Pandas Series with the Exponential Moving Average for the indicated price.

    Parameters
    ----------
    ohlc : pandas.DataFrame
        DataFrame with OHLC data
    price : str
        The price on which EMA will be calculated (Close by default)
    window : int
        Size of the moving window. (span in EMA, alpha = 2/(span + 1) for span >= 1)

    Returns
    -------
    pandas.Series
    """

    # implement exponential moving average
    output = ohlc[price].ewm(span=window, adjust=False).mean()
    output = output.rename(f'EMA{window}')

    return output


if __name__ == '__main__':
    pass



from . import DATA_DIR
import pandas as pd
import numpy as np
from os import path
from urllib.request import urlretrieve
from datetime import datetime, timedelta
from .scrapers import scrap_summary_table


def read_ohlcv_from_csv(file_path):
    """
    Reads and returns OHLCV data from CSV file.

    Parameters
    ----------
    file_path : str or os.path
        path to CSV file with OHLCV data
    Returns
    -------
    pandas.DataFrame
    """
    
    output = pd.read_csv(file_path, 
                         index_col='Date',
                         parse_dates=['Date'],
                         date_parser=lambda x: datetime.strptime(x, '%Y-%m-%d'))
    output['Volume'] = output['Volume'].astype(np.float64)
    return output


class StockQuotes:
    def __init__(self, ticker):
        super().__init__()
        self.ticker = ticker
        self._historical_ohlc = None

    @property
    def data(self):
        if self._historical_ohlc is None:
            self._historical_ohlc = self._get_data()
        return self._historical_ohlc

    @property
    def csv_file_path(self):
        return path.join(DATA_DIR, f'{self.ticker}_ohcl.csv')

    def download_ohlc_from_stooq(self):
        url = f'http://stooq.com/q/d/l/?i=d&s={self.ticker}'
        file_path, _ = urlretrieve(url)
        try:
            output = read_ohlcv_from_csv(file_path)
        except ValueError:  # Stooq: Exceeded the daily hits limit
            output = None
        return output

    def read_csv_file(self):
        return read_ohlcv_from_csv(self.csv_file_path)

    def _get_data(self):
        update_required = True  # assuming that update will be required
        output = pd.DataFrame()

        # time info 
        time_now = datetime.now()
        weekday_now = datetime.weekday(time_now)
        is_weekend = True if weekday_now in (5, 6) else False

        # calculate expected date for last OHLC data, consider only weekdays 
        # from Mo-Fr, assume that last update was one day earlier 
        delta_days = (weekday_now - 4) if is_weekend else 0
        expected_ohlc_time = time_now - timedelta(days=delta_days)
        last_ohlc_time = expected_ohlc_time - timedelta(days=1)

        # file with data for ticker exists
        if path.exists(self.csv_file_path):
            timestamp_now = datetime.timestamp(time_now)
            timestamp_up = path.getatime(self.csv_file_path)  # CSV file modification time

            # CSV updated within last 24 hours or it is weekend (no new data) 
            if (timestamp_now - timestamp_up < 24 * 3600) or is_weekend:
                output = self.read_csv_file()
                last_ohlc_time = output.iloc[-1].name

                if last_ohlc_time.date() == expected_ohlc_time.date() or (time_now.hour < 20 and not is_weekend):
                    update_required = False

        if update_required:
            # update CSV file and read data 
            new_output = self.download_ohlc_from_stooq()

            if not new_output.empty:
                # Updated data downloaded - save to CSV and update output 
                new_output.to_csv(self.csv_file_path)
                output = new_output
            else:
                # Update error (Stooq: Exceeded the daily hits limit) 
                pass
    
        if not output.empty:
            output.sort_index(ascending=True, inplace=True)
        
        return output


if __name__ == '__main__':
    pass



import requests
import pandas as pd
import re


def get_raw_summary_table(ticker):
    """
    Downloads and returns raw summary table from Stooq.

    Parameters
    ----------
    ticker : str
        ticker of a stock
    Returns
    -------
    pd.DataFrame
    """

    url = f'https://stooq.pl/q/g/?s={ticker}'
    html = requests.get(url).text

    # extracting table with summary
    raw_table = pd.read_html(html)[0]
    idx = raw_table.iloc[:, 0]
    raw_table.set_index(idx, inplace=True)

    return raw_table


def scrap_summary_table(ticker):
    """
    Scraps summary table with information about given stock ticker. Returns 
    dictionary with keys: Las - last price, Open - last open price, Volume - 
    last volume, EPS - EPS, P/E - P/E, P/BV - P/BV, Dividend yield % - dividend
    yield in percents. 

    Parameters
    ----------
    ticker : str
        ticker of a stock
    Returns
    -------
    dict
    """

    # get raw table    
    raw_table = get_raw_summary_table(ticker)

    # Stooq is Polish - translations needed
    keys = {
        'Last': 'Kurs',
        'Open': 'Otwarcie',
        'Volume': 'Wolumen',
        'EPS': 'EPS (ttm)',
        'P/E': 'C/Z (ttm)',
        'P/BV': 'C/WK',
        'Dividend yield %': 'Stopa dywidendy'
    }

    # creating and filling output dict 
    output_dict = dict()
    for k in keys:
        try:
            output_dict[k] = raw_table.loc[keys[k], 1]
        except KeyError:
            output_dict[k] = None

    # remove currency from price using regex
    m = re.search(r'\d+(\.\d+)?', output_dict['Last'])
    output_dict['Last'] = m.group(0)

    # remove % from dividend yield
    if output_dict['Dividend yield %']:
        output_dict['Dividend yield %'] = output_dict['Dividend yield %'][:-1] 

    for k in output_dict.keys():
        if output_dict[k]:
            output_dict[k] = float(output_dict[k])

    return output_dict


if __name__ == '__main__':
    pass



from . import DATA_DIR
from datetime import datetime
import pandas as pd
import requests
import os
import csv
from .scrapers import scrap_summary_table


class Fundamentals(dict):
    """
    Class Fundamentals
    """

    def __init__(self, ticker):
        super().__init__()
        self.ticker = ticker

    def get_fundamentals(self):
        """
        Scraps from Stooq.com fundamental information for given ticker.
        """

        update_required = True  # assuming that update will be required
        file_path = os.path.join(DATA_DIR, f'{self.ticker}_indicators.csv')

        if os.path.exists(file_path):
            timestamp_now = datetime.timestamp(datetime.now())
            timestamp_up = os.path.getatime(file_path)  # CSV file modification time

            if timestamp_now - timestamp_up < 24 * 3600:
                # do not update more often than once in 24 hours
                with open(file_path, 'r') as f:
                    reader = csv.DictReader(f)
                    try:
                        self.update(next(reader))
                    except StopIteration:
                        pass
                update_required = False
                # make sure that values are float
                for k in self.keys():
                    if self[k]:
                        self[k] = float(self[k])

        if update_required:
            # data older than 24 hours - update
            self.update(scrap_summary_table(self.ticker))
            with open(file_path, 'w') as f:
                writer = csv.DictWriter(f, self.keys())
                writer.writeheader()
                writer.writerow(self)

        if bool(self) is False:
            # no fundamental data (None or empty dict)
            self.update()

    
if __name__ == '__main__':
    pass
